C        PROGRAM SNIDE_POLARON

C*******************************************************************************
C                       COMPLEX SNIDE                                          *   
C          FOR NUMERICAL SOLVING OF INTEGRO-DIFFERENTIAL EQUATION              *
C Y''(x)+(Q(x)-LAMBDA*R(x))Y(x)+Y(x)**N1 (INTEGRAL(A,B)(K(x,x')Y(x')**N2 dx')=0*
C                  WITH BOUNDARY CONDITIONS                                    *
C         D1(LAMBDA,x)Y' + F1(LAMBDA,x)Y =0 FOR x=A,                           *
C         D2(LAMBDA,x)Y' + F2(LAMBDA,x)Y =0 FOR x=B                            *
C              AND WITH NORMALIZATION CONDITION                                *
C               INTEGRAL(A,B)(Y(x)**2 dx)=NORM                                 *
C*******************************************************************************
C                  TEST:POLARON PROBLEM                                        *
C              JINR COMMUNICATION P11-91-87, DUBNA, 1991.                      *
C              www.jinr.ru/programs/jinrlib/snide                              *
C*******************************************************************************

      IMPLICIT REAL*8(A-H,O-Z)
	PARAMETER(M=303)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION PM(M),V1(M),V2(M),SI1(M),VV(M),
     *SI(M),F(M),PM2(M),H(M),SL(M),Q1(M),R1(M)
      DIMENSION X(M),U(M),U0(M),XQ(60),Q(60),YK(60,60),
     *AS(10),HS(10),R(60)
    
C----------------------------------- CONSTANTS
      ZERO=0.D00 
      HALF=0.5D00
	ONE=1.D00
	TWO=2.D00
	THREE=3.D00
	FOUR=4.D00
	FIVE=5.D00
	SIX=6.D00 
	SEVEN=7.D00
	EIGHT=8.D00
	CNINE=9.D00
	TEN=10.D00
	PI=3.14159265D00

C----------------------------------- NH=STEP DIM. NPR=STEP ITER.
C------------------------------------ INPUT PARAMETERS FOR SNIDE

      N=M-2
      NQ=60
      NTAU=1
      NDEL=1
      NT=1
      NH=50
      NPR=1
C*** FOR INT=0  -- DIFFERENTIAL EQUATION
C*** FOR INT=1  -- INTEGRO-DIFFERENTIAL EQUATION   G(T)=1-EXP(-T)
C*** FOR INT=2  -- INTEGRO-DIFFERENTIAL EQUATION   G(T)=1

      INT=1
      T=.1D00
      EPS=.0001D00
      TAU0=.5D00
      NMAX=100
      N1=1
      N2=2 
  
	NORM=0
C FOR NFIX=1 -- MODIFICIRED PROCESS WITH SLT=SLTFIX
  	NFIX=0

C-------------------------------- PARAMETERS FOR 'POLARON PROBLEM'
      
C  NVAR=1  O 2   4 : 
C  A=AAA=0.6 NZ=0 ALPHA=0.6 MU=.1039 R=30 LAMBDA_NUM [1] = 0.10947 LAMBDA [16]=0.10953

      AAA=0.6D00
      NZ=0 
	ALPHA=.6d0
	SMU=0.1309D00 
	R=30.d0 
	LAMBDA=0.1D00 
	SL0=0.1D00
	SLT_OLD=0.10947D00
C      U0(I+1)=X(I)*DEXP(-X(I))
   
	AA=ZERO 
      BB=30.D00     
      HH=(BB-AA)/(N-1)
      DO 6 I=1,N
  6   X(I)=AA+HH*(I-1)
      X(1)=.000000001D00
      DO 111 I=1,N
      IP1=I+1
C      CALL SPL(X(I),U0(IP1),VV,F,N2,PM)

      U0(IP1)=X(I)*DEXP(-X(I)) ! FOR NVAR=1

      SI(I)=U0(IP1)      
 111  CONTINUE    


      PRINT *,' N=',N,' SLO=',SL0,' B=',BB,' H=',HH,' T=',T

      
C N=SSN=(A*ALPHA)**2[( 2/SMU - 1)**2-1]
	SSN=AAA*ALPHA*DSQRT((TWO/SMU - ONE)**2 - ONE) 

CORR 22.01.04
C      SSN=8.714195
C      SMU=TWO/(ONE+(SQRT(SSN*SSN/(AAA*AAA*ALPHA*ALPHA)+ONE)))
C      SL0=TWO*SMU*SMU/(AAA*AAA*(ONE-SMU))

      PRINT 1000
      PRINT 1010
      PRINT 1000
    
      PRINT *,' AAA=',AAA,' NZ=',NZ,' ALPHA=',ALPHA,
     1' SMU=',SMU,' SL0=',SL0

      SNP=ONE/(FOUR*PI)	 
      FF=ONE/DSQRT(SL0)
      GAMMA=(HALF/DSQRT(TWO))*DSQRT(DSQRT(HALF)*SL0/(PI*ALPHA))
 
      CALL HLM(N,X,SL,H)
      CALL PR(N,H,SL,SI,PM,V1,V2)
      U0(1)=PM(1)
      U0(M)=PM(N)
  
C-------------------------------------------- XQ(I),Q(I),R(I),YK(I,J)
      
      SQ2=DSQRT(TWO)
      AL=SMU*SQ2/DSQRT(ONE-SMU)
      ZZ=EIGHT*PI*SQ2*ALPHA
      AS(1)=ZERO
      AS(2)=HH*TEN
      AS(3)=BB
      AS(4)=TEN
      HS(1)=HH
      HS(2)=.1D00
      KS=3
      CALL SET(AS,HS,NQ,KS,XQ)
      XQ(1)=0.000000001D00
      DO 114 I=1,NQ
      Q(I)=ZERO
      R(I)=ONE
 114  CONTINUE
      DO 117 I=1,NQ
      DO 116 J=1,NQ
      IF(XQ(I).LE.XQ(J))GO TO 115
      SH=DEXP(AL*XQ(J))
      SH=(SH-ONE/SH)/TWO
      D4=DEXP(-AL*XQ(I))
      YK(I,J)=ZZ*(XQ(J)-SH*D4/AL)/(XQ(I)*XQ(J))
      GO TO 116
 115  SH=DEXP(AL*XQ(I))
      SH=(SH-ONE/SH)/TWO
      DX4=DEXP(-AL*XQ(J))
      YK(I,J)=ZZ*(XQ(I)-SH*DX4/AL)/(XQ(I)*XQ(J))
 116  CONTINUE
 117  CONTINUE

      SLTFIX=SL0

      CALL SNIDE(M,U0,SL0,X,NQ,YK,Q,R,INT,N1,N2,T,NDEL,TAU0,NTAU,NT,XQ,
     *EPS,NMAX,NH,NPR,NFIX,SLTFIX,NORM,SNP,U,SLT)

C--------------------INITIAL APPROACH OR ANALYTIC RESULT ------------

      SLA=TWO*SMU*SMU/((ONE-SMU)*AAA*AAA)


      DSA=DSQRT(SLA)
      NP1=N+1
      DO 16 I=2,NP1
C      U(I)=U(I)/S2SQP
 16   F(I)=U0(I)
      TO=0.
      II=0
      
      PRINT 1000
      PRINT 1001
      PRINT 1002
      DO 19 I=2,NP1
      IM1=I-1
      DEL=DABS(U(I)-F(I))
      IF(DEL.LE.TO)GO TO 17
      TO=DEL
      XMAX=X(IM1)
  17  CONTINUE
      IF(I.NE.(II*NH+2))GO TO 18
      II=II+1
      PRINT 1003,IM1,X(IM1),U(I),F(I),DEL
  18  CONTINUE
  19  CONTINUE
      
      PRINT 1004,TO,XMAX
      
      DL=DABS(SLT-SLA)
      PRINT 1005,SLT,DL
      PRINT 1000
	WRITE (*,1999)AAA,NZ,ALPHA,SMU,BB,SLT,SLT_OLD,SLA
	OPEN(3,FILE='SNIDE_OUT',STATUS='UNKNOWN')
      WRITE (3,1999)AAA,NZ,ALPHA,SMU,BB,SLT,SLT_OLD,SLA
	WRITE(3,*)' I    X(I)      PSI(I)     PSI0(I)'
      WRITE (3,2000)(I,X(I),U(I+1),F(I+1),I=1,N)
	CLOSE (3)

 1000 FORMAT(2X,70(1H*))
 1001 FORMAT(2X,'------------ RESULTS OF TEST  ------------')
 1002 FORMAT(2X,'I',6X,'ARGUMENT',8X,'NUMERICAL',8X,'INITIAL',9X,'DEL')
 1003 FORMAT(2X,I4,4(3X,E13.7))
 1004 FORMAT(2X,'DELTA MAX = ',E13.7,5X,'XMAX=',E13.7)
 1005 FORMAT(2X,'LAMBDA NUM.=',E13.7,2X,2X,'DELTA=',E13.7)
 1010 FORMAT(10X,'P O L A R O N     P R O B L E M')
 
 1999 FORMAT(2X,' a=',F3.1,'  NZ=',I1,' alpha=',D6.1,' mu=',D9.4,'  R=',
     ,D7.2,' lambda=',D10.5,' lambda[1]=',D10.5,' lambda[16]=',D10.5)
 2000 FORMAT(3E14.7)
      END
C
C****************************************************************************
C****************************************************************************
C
      SUBROUTINE SNIDE(M,U0,SL0,X,NQ,YK,Q,R,INT,N1,N2,T,NDEL,TAU0,NTAU,
     *NT,XQ,EPS,NMAX,NH,NPR,NFIX,SLTFIX,NORM,SNP,U,SLT)
      IMPLICIT REAL*8(A-H,O-Z)

	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION  PM(M),V1(M),V2(M),SI1(M),VV(M),
     *SI(M),F(M),PM2(M),H(M),SL(M),Q1(M),R1(M)
      DIMENSION X(M),U(M),U0(M),XQ(NQ),Q(NQ),YK(NQ,NQ),H1(60),
     *SL1(60),R(NQ),DEF(10)

      N=M-2
      NP1=N+1
      IF(INT.EQ.1)SJN=ONE
      IF(INT.EQ.0)SJN=0.
      IF(INT.EQ.2)SJN=ONE
C------------------------------------------------ INTERPOLATION OF Q,R
      CALL HLM(NQ,XQ,SL1,H1)
      CALL PR(NQ,H1,SL1,Q,PM,VV,SI)
      DO 80 I=1,N
 80   CALL SPL(X(I),Q1(I),XQ,Q,NQ,PM)
      CALL PR(NQ,H1,SL1,R,PM,VV,SI)
      DO 90 I=1,N
 90   CALL SPL(X(I),R1(I),XQ,R,NQ,PM)
      
      II=0
      PRINT 3000
      PRINT 3001
      DO 30 I=1,N
      IF(I.NE.(II*NH+1))GO TO 30
      II=II+1
      PRINT 3002,I,X(I),U0(I+1),Q1(I)
  30  CONTINUE
      PRINT 3003,U0(1),U0(M),TAU0,EPS
      PRINT 3004,M,NT,NTAU,NMAX,INT,NH,NPR,NDEL
      PRINT 3005,X(1),X(N),SL0,NQ
      NIT=0
      IPR=0
      TAU=TAU0
      TP=TAU0
      SLT=SL0
      SLF=SLTFIX
      N2M1=N2-1
      N2M2=N2-2
      N1M1=N1-1
      DDD=ZERO
      D1=ZERO
      D0=ZERO
      DO 1 I=1,M
      U(I)=U0(I)
      V1(I)=ZERO
      V2(I)=ZERO
      SI(I)=ZERO
      SI1(I)=ZERO
      VV(I)=ZERO
 1    CONTINUE
      DELT=ONE/NT
      ITAU=ONE
      SQSNP=DSQRT(SNP)
      CALL HLM(N,X,SL,H)
      DO 56 I=2,NP1
 56   VV(I)=U(I)*U(I)
C-------------------------------------------------- BEGIN OF ITERATION
 15   CONTINUE
      DO 763 I=2,NP1
 763  IF(U(I).EQ.ZERO)U(I)=U(I)+1.E-13
      PRINT 1003
      CALL G(T,GT,GT1,INT)
      CALL DFT(SLT,SLTFIX,D10,D11,D20,D22,F10,F11,F20,F22,DT10,DT11,
     *DT20,DT22,FT10,FT11,FT20,FT22,X(1),X(N))
C------------------------------------------------ NORMALIZATION OF U(I)
      VV(1)=TWO*U(1)*U(2)
      VV(M)=TWO*U(NP1)*U(M)
      CALL P1SPL(M,H,SL,VV,PM,PM2,SI)
      CALL SINT(M,H,PM,VV,S1)
      IF(NORM.EQ.0)GO TO 152
      SQ=DSQRT(S1)
      DO 52 I=1,M
  52  U(I)=U(I)*SQSNP/SQ
 152  CONTINUE
C----------------------------------------------- CALCULATION OF DELTA
      CALL DELTA(INT,M,SLT,Q1,U,YK,SI,NQ,VV,F,PM,PM2,SI1,R1,N2,
     *XQ,X,H,SL,H1,SL1)
      IF(NIT.EQ.(NPR*IPR))PRINT 1001
      II=0
      NZERO=0
      DO 818 I=2,N
      IF(U(I)*U(I+1).GE.0.)GO TO 818
      NZERO=NZERO+1
      PRINT 7000,X(I)
  818 CONTINUE
      DO 18 I=1,N
      I1=I+1
      VV(I)=DABS(PM2(I)+SJN*SI(I)*U(I1)**N1)
      F(I)=DABS(PM2(I)+SJN*SI(I)*U(I1)**N1*GT)
C      PRINT 5000,I,X(I),SI(I),PM2(I),Q1(I),VV(I),U(I1)
C 5000 FORMAT(2X,I4,6(2X,E13.7))
      IF(NIT.NE.(NPR*IPR))GO TO 50
      IF(I.NE.(NH*II+1))GO TO 51
      II=II+1
      PRINT 1000,I,X(I),U(I1),F(I),SI(I)
  51  CONTINUE
  50  CONTINUE
      IF(F(I).LE.D1)GO TO 118
      D1=F(I)
      XMAX1=X(I)
  118 CONTINUE
      IF(VV(I).LE.DDD)GO TO 119
      DDD=VV(I)
      XMAX=X(I)
 119  CONTINUE
  18  CONTINUE
      IF(NIT.EQ.(NPR*IPR))IPR=IPR+1
      PRINT 4004,DDD,XMAX1,NZERO
      IF(NDEL.EQ.1)GO TO 895
      DO 896 I=1,N
 896  VV(I)=VV(I)*VV(I)
      CALL PR(N,H,SL,VV,PM,V1,SI1)
      DO 897 I=1,NP1
      NI=N+2-I
 897  VV(NI+1)=VV(NI)
      CALL SINT(M,H,PM,VV,S)
C     DINT0=DSQRT(S)
      D1=DABS(S)
 895  CONTINUE
      PRINT 1002,NIT,D1,XMAX,TAU
      PRINT 1027,SLT
      DEL1=(D10+SJN*D11)*U(1)+(F10+SJN*F11)*U(2)
      DEL2=(D20+SJN*D22)*U(M)+(F20+SJN*F22)*U(NP1)
      PRINT 2000,DEL1,DEL2

      IF(DDD.LE.EPS)GO TO 12
      IF(D1.LE.EPS)GO TO 12
      IF(NIT.GE.NMAX)GO TO 13
C---------------------------------------- SOLUTION OF BOUNDARY PROBLEMS
      IF(INT.EQ.0)GO TO 271
      DO 77 I=1,NQ
      DO 91 J=1,NQ
  91  F(J)=YK(I,J)
      CALL PR(NQ,H1,SL1,F,PM,VV,V1)
      DO 8 J=2,NP1
      CALL SPL(X(J-1),VV(J),XQ,F,NQ,PM)
  8   VV(J)=VV(J)*V2(J)*U(J)**N2M1*N2
      VV(1)=((PM(1)*V2(2)+F(1)*V2(1))*U(2)**N2M1+
     *V2(2)*F(1)*U(2)**N2M2*N2M1*U(1))*N2
      VV(M)=((PM(NQ)*V2(NP1)+F(NQ)*V2(M))*U(NP1)**N2M1+
     *U(N)**N2M2*N2*U(M)*F(NQ)*V2(NP1))*N2
      CALL P1SPL(M,H,SL,VV,PM,F,V1)
      CALL SINT(M,H,PM,VV,SI1(I))
  77  CONTINUE
      CALL PR(NQ,H1,SL1,SI1,PM,F,VV)
      DO 177 I=1,N
      CALL SPL(X(I),VV(I),XQ,SI1,NQ,PM)
      V1(I)=-((GT1+GT)*SI(I)+GT*VV(I))*U(I+1)**N1-PM2(I)
  177 CONTINUE
  271 CONTINUE
      IF(NFIX.NE.1)SLF=SLT
      DO 273 I=1,N
 273  VV(I)=Q1(I)-SLF*R1(I)+GT*SI(I)*N1*U(I+1)**N1M1
      G1=-GT1*(D11*U(1)+F11*U(2))-(D10+GT*D11)*U(1)-(F10+GT*F11)*U(2)
      G2=-GT1*(D22*U(M)+F22*U(NP1))-(D20+GT*D22)*U(M)-
     *(F20+GT*D22)*U(NP1)
      DD1=D10+GT*D11
      D2=D20+GT*D22
      F1=F10+GT*F11
      F2=F20+GT*F22
      IF(INT.EQ.0)GO TO 71
      CALL DIFSPL(X,SL,M,H,V1,VV,F1,F2,DD1,D2,G1,G2,V2,PM2,SI1)
  71  CONTINUE
      G1=-(FT10+GT*FT11)*U(2)-(DT10+GT*DT11)*U(1)
      G2=-(FT20+FT22*GT)*U(NP1)-(DT20+GT*DT22)*U(M)
      DO 9 I=1,N
  9   PM(I)=U(I+1)*R1(I)
      CALL DIFSPL(X,SL,M,H,PM,VV,F1,F2,DD1,D2,G1,G2,V1,PM2,SI1)
C---------------------------------------------- CALCULATION OF 'SMU'
      DO 4 I=2,NP1
  4   VV(I)=U(I)*U(I)
      VV(1)=TWO*U(1)*U(2)
      VV(M)=TWO*U(M)*U(NP1)
      CALL P1SPL(M,H,SL,VV,PM,PM2,SI)
      CALL SINT(M,H,PM,VV,S1)
      DO 5 I=2,NP1
  5   VV(I)=U(I)*V2(I)
      VV(1)=V2(1)*U(2)+V2(2)*U(1)
      VV(M)=V2(NP1)*U(M)+V2(M)*U(NP1)
      CALL P1SPL(M,H,SL,VV,PM,PM2,SI)
      CALL SINT(M,H,PM,VV,S2)
      DO 6 I=2,NP1
  6   VV(I)=U(I)*V1(I)
      VV(1)=V1(1)*U(2)+V1(2)*U(1)
      VV(M)=V1(NP1)*U(M)+V1(M)*U(NP1)
      CALL P1SPL(M,H,SL,VV,PM,PM2,SI)
      CALL SINT(M,H,PM,VV,S3)
      SMU=(.5*(-S1+SNP)-S2)/S3
      PRINT 1026,SMU
C------------------------------------------- CALCULATION OF V(I)
      DO 222 I=1,M
      V2(I)=V1(I)*SMU+V2(I)
  222 CONTINUE
C------------------------------------------------------- TAU=4,5
      IF(NTAU.LE.3)GO TO 403
      IF(NTAU.EQ.4)TAU=ONE
 299  IF(NTAU.EQ.5)TAU=DELT*ITAU
      SLT1=SLT+TAU*SMU
      DO 220 I=1,M
      V1(I)=U(I)+TAU*V2(I)
      VV(I)=V1(I)*V1(I)
  220 CONTINUE
      VV(1)=TWO*V1(1)*V1(2)
      VV(M)=TWO*V1(NP1)*V1(M)
      CALL P1SPL(M,H,SL,VV,PM,PM2,SI)
      CALL SINT(M,H,PM,VV,SS)
      IF(NORM.EQ.0)GO TO 283
      SQ=DSQRT(SS)
      DO 252 I=1,M
  252 V1(I)=V1(I)/SQ
  283 CONTINUE
      CALL DELTA(INT,M,SLT,Q1,V1,YK,SI,NQ,VV,F,PM,PM2,SI1,R1,N2,
     *XQ,X,H,SL,H1,SL1)
      DO 218 I=1,N
      F(I)=DABS(PM2(I)+SJN*SI(I)*U(I+1)**N1)
 218  CONTINUE
      IF(NDEL.EQ.2)GO TO 219
      DK1=0.
      DO 319 I=1,N
      IF(F(I).GT.DK1)DK1=F(I)
 319  XMAX=X(I)
      DEF(ITAU)=DK1
      GO TO 519
 219  DO 419 I=1,N
      F(I)=F(I)*F(I)
 419  CONTINUE
      CALL PR(N,H,SL,F,PM,VV,SI1)
      DO 804 I=1,NP1
      NI=N+2-I
      F(NI+1)=F(NI)
  804 CONTINUE
      CALL SINT(M,H,PM,F,S)
C     DINT1=DSQRT(S)
      DINT1=DABS(S)
      DEF(ITAU)=DINT1
 519  CONTINUE
      ITAU=ITAU+1
      IF(ITAU.LE.NT)GO TO 299
      D1=DEF(1)
      IF(NTAU.EQ.4)GO TO 403
      DO 719 I=2,NT
      IF(DEF(I).GE.D1)GO TO 718
      D1=DEF(I)
      TP=I*DELT
 718  CONTINUE
 719  CONTINUE
C---------------------------------------------------- TAUKA ----
 403  CALL TAUKA(NTAU,TP,D0,D1,TAU0,TAU)
      D0=D1
      D1=ZERO
      SLT=SLT+TAU*SMU
      DO 20 I=1,M
      U(I)=U(I)+TAU*V2(I)
      VV(I)=U(I)*U(I)
  20  CONTINUE
C      IF(SMU.GT.100.)GO TO 19
      TP=TAU
      ITAU=1
      T=T+TAU
      DDD=ZERO
      NIT=NIT+1
      GO TO 15
C-------------------------------------------------- END OF ITERATION
  12  PRINT 1005
      GO TO 19
  13  PRINT 1006
  19  CONTINUE
C     PRINT 3050
C     DO 880 I=2,NP1,NH
C     PRINT 3052,X(I-1),U(I)
C880  CONTINUE
 3050 FORMAT(2X,'------------- R E S U L T -------------------')
 3052 FORMAT(2X,E13.7,3X,E13.7)
 1005 FORMAT(2X,'--------------- END: DELTA<EPS-------------')
 1006 FORMAT(2X,'--------------- END: NIT>NMAX--------------')
 
 1003 FORMAT(2X,70(1H\))
 1000 FORMAT(2X,I4,4(3X,E13.7))
 1002 FORMAT('  NIT=',I4,' D1=',E13.7,' XMAX=',E13.7,' TAU=',E13.7)
 1001 FORMAT(2X,'I',7X,'X',15X,'U',15X,'DEL',15X,'SI')
 3000 FORMAT(2X,'---------------  INPUT PARAMETERS  ---------------')
 3001 FORMAT(2X,'I',5X,'X(I)',15X,'U0(I)',15X,'Q(I)')
 3002 FORMAT(2X,I4,3X,3(E13.7,6X))
 3003 FORMAT('  U1=',E13.7,' U2=',E13.7,' TAU0=',E13.7,' EPS=',E13.7)
 3004 FORMAT(2X,'M=',I4,1X,'N=',I4,1X,'NTAU=',I1,1X,'NMAX=',I4,1X,
     *'INT=',I1,1X,'NH=',I4,1X,'NPR=',I4,1X,'NDEL=',I1)
 7000 FORMAT('   XZERO=',E13.7)
 4004 FORMAT(2X,'DDD=',E13.7,2X,'XMAX1=',E13.7,2X,'NZERO=',I3)
 1027 FORMAT(2X,'============= SLT=',E13.7,'===========')
 2000 FORMAT(2X,' DEL1=',E13.7,'   DEL2=',E13.7)
 1026 FORMAT(2X,'=========== SMU=',E13.7,'=================')
 3005 FORMAT(2X,' A=',E13.7,2X,'B=',E13.7,2X,'SL0=',E13.7,2X,'NQ=',I4)
      RETURN
      END
C
C******************************************************************************
C
      SUBROUTINE G(T,GT,GT1,INT)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      IF(INT.EQ.1)GO TO 1
      IF(INT.EQ.2)GO TO 2
      GT=0.
      GT1=0.
      GO TO 3
  2   GT=ONE
      GT1=0.
      GO TO 3
  1   CONTINUE
      GT1=DEXP(-T)
      GT=ONE-GT1
  3   CONTINUE
C      PRINT 1000,INT,T,GT,GT1
 1000 FORMAT('  INT=',I1,' T=',E13.7,' GT=',E13.7,' GT1=',E13.7)
      RETURN
      END
C
C******************************************************************************
C
      SUBROUTINE TAUKA(NTAU,TP,D0,D1,TAU0,TAU)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      IF(NTAU.EQ.2)GO TO 2
      IF(NTAU.EQ.3)GO TO 3
      IF(NTAU.EQ.4)GO TO 4
      IF(NTAU.EQ.1)TAU=TAU0
      IF(NTAU.EQ.5)TAU=TP
      GO TO 1
  3   IF(D1.LT.D0)TAU=DMIN1(ONE,TP*(D0/D1))
      IF(D1.GE.D0)TAU=DMAX1(TAU0,TP*(D0/D1))
      GO TO 1
  2   IF(D1.LT.D0)TAU=DMIN1(ONE,2.D00*TP)
      IF(D1.GE.D0)TAU=DMAX1(TAU0,TP/2.D00)
      GO TO 1
  4   TAU=DMAX1(TAU0,D0/(D0+D1))
  1   CONTINUE
      RETURN
      END
C
C********************************************************************************
C
      SUBROUTINE DFT(SLT,SLTFIX,D10,D11,D20,D22,F10,F11,F20,F22,
     *DT10,DT11,DT20,DT22,FT10,FT11,FT20,FT22,A,B)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      D10=0.D00
      D11=0.D00

      D20=1.D00
	D20=0.D00

      D22=0.D00
	F10=1.D00

	F11=0.D00
	F11=1.D00

      F20=.25D00
	F20=1.D00

	F22=0.D00
	F22=1.D00
      DT10=0.D00
      DT11=0.D00
      DT20=0.D00
      DT22=0.D00
      FT10=0.D00
      FT11=0.D00
      FT20=0.D00
      FT22=0.D00
      RETURN
      END
C
C****************************************************************************
C
      SUBROUTINE DIFSPL(X,SL,N1,H,F,Q,F1,F2,D1,D2,G1,G2,V,QQ,U)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION X(N1),SL(N1),F(N1),Q(N1),V(N1),QQ(N1),U(N1),H(N1)
      N=N1-2
      NM1=N-1
      NP1=N+1
      NM2=N-2
      IF(D1.LE.0.00000001)GO TO 100
      K1=1
      QQ(1)=-F1/D1
      U(1)=G1/D1
C------------------------------------------------- I=2
      A=-SIX/H(1)
      B=A/H(1)+TWO*Q(1)
      C=-A/H(1)+SL(1)*Q(2)
      D=TWO*F(1)+SL(1)*F(2)
      P=A*QQ(1)+B
      QQ(2)=-C/P
      U(2)=(D-A*U(1))/P
C------------------------------------------------- I=3 (D1.NE.0.)
      A=SIX/(H(1)*(H(2)+H(1)))+(ONE-SL(2))*Q(1)
      B=-SIX/(H(1)*H(2))+TWO*Q(2)
      C=SIX/(H(2)*(H(1)+H(2)))+SL(2)*Q(3)
      D=(ONE-SL(2))*F(1)+TWO*F(2)+SL(2)*F(3)
      P=A*QQ(2)+B
      QQ(3)=-C/P
      U(3)=(D-A*U(2))/P
      GO TO 101
C--------------------------------------------------- I=3 (D1=0.)
  100 K1=3
      V(2)=G1/F1
      A=SIX/(H(1)*(H(2)+H(1)))+(ONE-SL(2))*Q(1)
      B=-SIX/(H(1)*H(2))+TWO*Q(2)
      C=SIX/(H(2)*(H(1)+H(2)))+SL(2)*Q(3)
      D=(ONE-SL(2))*F(1)+TWO*F(2)+SL(2)*F(3)-A*V(2)
      QQ(3)=-C/B
      U(3)=D/B
C--------------------------------------------------- I=4,N-1
  101 CONTINUE
      DO 1 I=4,NM1
      I2=I-2
      I1=I-1
      A=SIX/(H(I2)*(H(I1)+H(I2)))+(ONE-SL(I1))*Q(I2)
      B=-SIX/(H(I2)*H(I1))+TWO*Q(I1)
      C=SIX/(H(I1)*(H(I2)+H(I1)))+SL(I1)*Q(I)
      D=(ONE-SL(I1))*F(I2)+TWO*F(I1)+SL(I1)*F(I)
      P=A*QQ(I1)+B
      QQ(I)=-C/P
      U(I)=(D-A*U(I1))/P
  1   CONTINUE
C---------------------------------------------------- I=N (D2.NE.0.)
      IF(D2.EQ.0.)GO TO 102
      K2=N1
      A=SIX/(H(NM2)*(H(NM1)+H(NM2)))+(ONE-SL(NM1))*Q(NM2)
      B=-SIX/(H(NM2)*H(NM1))+TWO*Q(NM1)
      C=SIX/(H(NM1)*(H(NM2)+H(NM1)))+SL(NM1)*Q(N)
      D=(ONE-SL(NM1))*F(NM2)+TWO*F(NM1)+SL(NM1)*F(N)
      P=A*QQ(NM1)+B
      QQ(N)=-C/P
      U(N)=(D-A*U(NM1))/P
C---------------------------------------------------- I=N+1
      C=SIX/H(NM1)
      A=C/H(NM1)+(ONE-SL(N))*Q(NM1)
      B=-C/H(NM1)+TWO*Q(N)
      D=(ONE-SL(N))*F(NM1)+TWO*F(N)
      P=A*QQ(N)+B
      QQ(NP1)=-C/P
      U(NP1)=(D-A*U(N))/P
C---------------------------------------------------- I=N+2
      P=F2*QQ(NP1)+D2
      QQ(N1)=0.
      U(N1)=(G2-F2*U(NP1))/P
      GO TO 106
C---------------------------------------------------- I=N (D2=0.)
  102 K2=N
      V(NP1)=G2/F2
      A=SIX/(H(NM2)*(H(NM1)+H(NM2)))+(ONE-SL(NM1))*Q(NM2)
      B=-SIX/(H(NM2)*H(NM1))+TWO*Q(NM1)
      C=SIX/(H(NM1)*(H(NM2)+H(NM1)))+SL(NM1)*Q(N)
      D=(ONE-SL(NM1))*F(NM2)+TWO*F(NM1)+SL(NM1)*F(N)-C*V(NP1)
      P=A*QQ(NM1)+B
      QQ(N)=0.
      U(N)=(D-A*U(NM1))/P
C-------------------------------------------------------
  106 CONTINUE
      V(K2)=U(K2)
      K3=K2-K1
      DO 2 I=1,K3
      K2I=K2-I
      V(K2I)=V(K2I+1)*QQ(K2I)+U(K2I)
  2   CONTINUE
C-----------------------------------------------------
      IF(K1.NE.3)GO TO 103
      A=-SIX/H(1)
      B=A/H(1)+TWO*Q(1)
      C=-A/H(1)+SL(1)*Q(2)
      D=TWO*F(1)+SL(1)*F(2)
      V(1)=(D-C*V(3)-B*V(2))/A
C--------------------------------------------------------
  103 CONTINUE
      IF(K2.NE.N)GO TO 104
      C=SIX/H(NM1)
      A=C/H(NM1)+(ONE-SL(N))*Q(NM1)
      B=-C/H(NM1)+TWO*Q(N)
      D=(ONE-SL(N))*F(NM1)+TWO*F(N)
      V(N1)=(D-A*V(N)-B*V(NP1))/C
  104 CONTINUE
      RETURN
      END
C
C******************************************************************************
C
      SUBROUTINE HLM(N,X,SL,H)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION SL(N),H(N),X(N)
      N1=N-1
      DO 1 I=1,N1
  1   H(I)=X(I+1)-X(I)
      DO 2 I=2,N1
      SL(I)=H(I)/(H(I)+H(I-1))
  2   CONTINUE
      SL(1)=1.D00
      H(N)=0.D00
      SL(N)=0.D00
      RETURN
      END
C
C******************************************************************************
C
      SUBROUTINE P1SPL(NN,H,SL,V,PM,U,Q)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION H(NN),SL(NN),V(NN),PM(NN),U(NN),Q(NN)
      U(1)=V(1)
      Q(1)=0.
      N=NN-2
      N1=N-1
      DO 1 I=2,N1
      IM1=I-1
      IP1=I+1
      SM=ONE-SL(I)
      D=(SM*((V(I+2)-V(IP1))/H(I))+SL(I)*((V(IP1)-V(I))/H(IM1)))*THREE
      P=SL(I)*Q(IM1)+TWO
      Q(I)=-(ONE-SL(I))/P
      U(I)=(D-SL(I)*U(IM1))/P
  1   CONTINUE
C--------------------------------------------------------
      PM(N)=V(N+2)
      DO 2 I=1,N1
      NI=N-I
      PM(NI)=Q(NI)*PM(NI+1)+U(NI)
  2   CONTINUE
      RETURN
      END
C
C************************************************************************
C
      SUBROUTINE PR(N,H,SL,F,PM,U,Q)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION H(N),SL(N),F(N),PM(N),U(N),Q(N)
      N1=N-2
      G1=H(1)/H(2)
      GN=H(N-1)/H(N1)
      FF=(F(3)-F(2))/H(2)
      FF1=(F(2)-F(1))/H(1)
      D1=(ONE-SL(2))*FF+SL(2)*FF1
      D=D1+TWO*G1*FF
      P=ONE+G1
      Q(2)=-G1/P
      U(2)=D/P
C-------------------------------------------------------
      DO 1 I=3,N1
      I1=I-1
      SM=ONE-SL(I)
      D=(SM*((F(I+1)-F(I))/H(I))+SL(I)*((F(I)-F(I1))/H(I1)))*THREE
      P=SL(I)*Q(I1)+TWO
      Q(I)=-(ONE-SL(I))/P
      U(I)=(D-SL(I)*U(I1))/P
  1   CONTINUE
C--------------------------------------------------------
      I=N-1
      I1=I-1
      SM=ONE-SL(I)
      FFN=(F(I)-F(I1))/H(I1)
      FFN1=(F(N)-F(I))/H(I)
      D1=SM*FFN1+SL(I)*FFN
      D=D1+TWO*GN*FFN
      P=GN*Q(I1)+ONE+GN
C--------------------------------------------------------
      PM(I)=(D-GN*U(I1))/P
      DO 2 I=2,N1
      NI=N-I
      PM(NI)=Q(NI)*PM(NI+1)+U(NI)
  2   CONTINUE
      G12=G1*G1
      GN2=GN*GN
      PM(1)=TWO*(FF1-G12*FF)+G12*PM(3)-(ONE-G12)*PM(2)
      PM(N)=TWO*(FFN1-GN2*FFN)-(ONE-GN2)*PM(N-1)+GN2*PM(N-2)
      RETURN
      END
C
C******************************************************************************
C
      SUBROUTINE SINT(NN,H,PM,V,SI)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION H(NN),PM(NN),V(NN)
      S1=0.
      S2=0.
      N1=NN-3
      N2=NN-1

      DO 1 I=1,N1
      I1=I+1
      S1=S1+(PM(I)-PM(I1))*H(I)*H(I)
      S2=S2+(V(I1)+V(I+2))*H(I)
   1  CONTINUE
      SI=S1/12.D00+S2/TWO
	RETURN
      END
C
C*******************************************************************************
C
      SUBROUTINE PR2SPL(PM,V,H,NN,PM2)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION PM(NN),V(NN),H(NN),PM2(NN)
      N=NN-2
      PM2(1)=SIX*(V(3)-V(2))/(H(1)*H(1))-FOUR*PM(1)/H(1)-TWO*PM(2)/H(1)
      DO 1 I=2,N
      I1=I-1
      ZH2=H(I1)*H(I1)
      PM2(I)=-SIX*(V(I+1)-V(I))/ZH2+TWO*PM(I1)/H(I1)+FOUR*PM(I)/H(I1)
  1   CONTINUE
      RETURN
      END
C
C*******************************************************************************
C
      SUBROUTINE SPL(XX,YY,X,Y,N,PM)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION PM(N),X(N),Y(N)
      DO 1 I=2,N
      IF(XX.LE.X(I))GO TO 2
  1   CONTINUE
  2   CONTINUE
      I=I-1
      H=X(I+1)-X(I)
      T=(XX-X(I))/H
      TM1=(ONE-T)*(ONE-T)
      TT=T*T
      I1=I+1
      YY=Y(I)*TM1*(ONE+TWO*T)-PM(I1)*H*TT*(ONE-T)
     *+Y(I1)*(THREE-TWO*T)*TT+PM(I)*H*T*TM1
      RETURN
      END
C
C*******************************************************************************
C
      SUBROUTINE SET(A,H,N,K,X)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION A(K),H(K),X(N)
      PRINT 1000
      PRINT 1001
      MP=1
      MS=0
      K1=K-1
      DO 1 I=2,K1
      I1=I-1
      M=INT((A(I)-A(I1))/H(I1)+.5)
      MS=MS+M
      DO 2 J=MP,MS
  2   X(J)=A(I1)+(J-MP)*H(I1)
      PRINT 1002,A(I1),A(I),H(I1),M
      MP=MP+M
 1    CONTINUE
      M=N-MP+1
      H(K1)=(A(K)-A(K1))/M
      DO 3 J=MP,N
  3   X(J)=A(K1)+(J-MP+1)*H(K1)
      X(N)=A(K)
      PRINT 1002,A(K1),A(K),H(K1),M

 1000 FORMAT(5X,'============== SETKA ==================')
 1001 FORMAT(5X,'A',15X,'B',15X,'H',15X,'M')
 1002 FORMAT(5X,3(E13.7,2X),I4)
      RETURN
      END
C
C*******************************************************************************
C
      SUBROUTINE DELTA(INT,NN,SLT,Q1,U,YK,SI,NQ,VV,F,PM,PM2,SI1,R1,N2,
     *XQ,X,H,SL,H1,SL1)
      IMPLICIT REAL*8(A-H,O-Z)
	COMMON /CONST/ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,
     *CNINE,TEN,PI
      DIMENSION Q1(NN),U(NN),YK(NQ,NQ),VV(NN),F(NN),PM2(NN),R1(NN),
     *SI1(NN),XQ(NQ),X(NN),H(NN),SL(NN),H1(NQ),SL1(NQ),SI(NN),PM(NN)
      N=NN-2
      N1=NN-1
      N2M1=N2-1
      CALL P1SPL(NN,H,SL,U,PM,PM2,SI)
      CALL PR2SPL(PM,U,H,NN,PM2)
C      CALL PR(N,H,SL,PM,PM2,VV,SI)
      IF(INT.EQ.0)GO TO 73
      DO 10 I=1,NQ
      DO 92 J=1,NQ
  92  SI1(J)=YK(I,J)
      CALL PR(NQ,H1,SL1,SI1,PM,VV,SI)
      VV(1)=PM(1)*U(2)**N2+SI1(1)*U(1)*U(2)**N2M1*N2
      VV(NN)=PM(NQ)*U(N1)**N2+SI1(NQ)*U(NN)*U(N1)**N2M1*N2
      DO 11 J=2,N1
      CALL SPL(X(J-1),VV(J),XQ,SI1,NQ,PM)
  11  VV(J)=U(J)**N2*VV(J)
      CALL P1SPL(NN,H,SL,VV,PM,SI,SI1)
      CALL SINT(NN,H,PM,VV,F(I))
  10  CONTINUE
      CALL PR(NQ,H1,SL1,F,PM,VV,SI)
      DO 94 I=1,N
      CALL SPL(X(I),SI(I),XQ,F,NQ,PM)
  94  CONTINUE
  73  CONTINUE
      DO 18 I=1,N
      PM2(I)=PM2(I)+(Q1(I)-SLT*R1(I))*U(I+1)
  18  CONTINUE
      RETURN
      END
C
C*******************************************************************************
CCC              END OF COMPLEX SNIDE
C*******************************************************************************
C